package com.atlassian.confluence.rpc.soap.services;

/**
 * A Soap service holding functionality for Confluence Administrators.
 */

import com.atlassian.confluence.importexport.ImportExportManager;
import com.atlassian.confluence.importexport.ImportExportException;
import com.atlassian.confluence.importexport.DefaultExportContext;
import com.atlassian.confluence.rpc.RemoteException;
import com.atlassian.confluence.rpc.soap.beans.RemoteServerInfo;
import com.atlassian.confluence.rpc.soap.beans.RemoteClusterInformation;
import com.atlassian.confluence.rpc.soap.beans.RemoteNodeStatus;
import com.atlassian.confluence.user.AuthenticatedUserThreadLocal;
import com.atlassian.confluence.util.GeneralUtil;
import com.atlassian.confluence.setup.BootstrapManager;
import com.atlassian.confluence.security.GateKeeper;
import com.atlassian.confluence.search.lucene.ConfluenceIndexManager;
import com.atlassian.confluence.spaces.SpaceManager;
import com.atlassian.confluence.cluster.ClusterManager;
import com.atlassian.confluence.cluster.NodeStatusFactory;
import com.atlassian.confluence.cluster.NodeStatus;
import com.atlassian.user.User;
import com.tangosol.net.Member;

import java.io.IOException;
import java.util.*;

import org.apache.log4j.Category;
import bucket.user.UserAccessor;

public class AdminSoapService
{
    public static final Category log = Category.getInstance(AdminSoapService.class);

    private ConfluenceIndexManager indexManager;
    private ImportExportManager importExportManager;
    private BootstrapManager bootstrapManager;
    private SpaceManager spaceManager;
    private GateKeeper gateKeeper;
    SoapServiceHelper soapServiceHelper;
    private UserAccessor userAccessor;
    private ClusterManager clusterManager;

    public void setUserAccessor(UserAccessor userAccessor)
    {
        this.userAccessor = userAccessor;
    }

    public void setSpaceManager(SpaceManager spaceManager)
    {
        this.spaceManager = spaceManager;
    }

    public void setImportExportManager(ImportExportManager importExportManager)
    {
        this.importExportManager = importExportManager;
    }


    /**
     * @param user - the current authenticated user
     * @return true if the user has the SpacePermission.ADMINISTER_CONFLUENCE_PERMISSION, otherwise false
     * @throws RemoteException
     */
    public boolean isConfluenceAdministrator(User user) throws RemoteException
    {
        return GeneralUtil.isSuperUser(user);
    }

    public String exportSite(boolean exportAttachments) throws RemoteException
    {
        String downloadPath = null;

        //1. get the user for the export
        User user = AuthenticatedUserThreadLocal.getUser();

        //2. permission check
        if (!isConfluenceAdministrator(user))
            throw new RemoteException("Authenticated user [" + user + "] is not a Confluence Administrator.");

        //3. export
        try
        {
            DefaultExportContext exportContext = DefaultExportContext.getXmlBackupInstance();
            exportContext.setExportAttachments(exportAttachments);
            exportContext.setExportHierarchy(false);
            exportContext.setExportComments(false);

            String archivePath = importExportManager.exportAs(exportContext);
            downloadPath = importExportManager.prepareDownloadPath(archivePath);

            gateKeeper.addKey(downloadPath, user);
        }
        catch (ImportExportException e)
        {
            return "Could not export space: " + e;
        }
        catch (IOException e)
        {
            return "Could not export space: " + e;
        }

        return bootstrapManager.getBaseUrl() + downloadPath;
    }

    public void setBootstrapManager(BootstrapManager bootstrapManager)
    {
        this.bootstrapManager = bootstrapManager;
    }

    public void setGateKeeper(GateKeeper gateKeeper)
    {
        this.gateKeeper = gateKeeper;
    }

    public void setSoapServiceHelper(SoapServiceHelper soapServiceHelper)
    {
        this.soapServiceHelper = soapServiceHelper;
    }

    public void setIndexManager(ConfluenceIndexManager indexManager)
    {
        this.indexManager = indexManager;
    }

    public RemoteServerInfo getServerInfo() throws RemoteException
    {
        // As of Confluence 2.0, we will allow the user to view server information even if they're not an admin (CONF-4566)
        return new RemoteServerInfo();
    }

    //---( Indexing related Remote API. This can be moved into its own service when it becomes a little larger.)---

    /**
     * Support remote interface for flushing the index queue.
     *
     */
    public boolean flushIndexQueue() throws RemoteException
    {
        soapServiceHelper.assertCanAdminister();
        while (indexManager.isReIndexing())
        {
            System.out.println("reindexing");
            try
            {
                Thread.sleep(1000);
            } catch (InterruptedException e)
            {
                //
            }
        }
        indexManager.flushQueue();
        return true;
    }

    /**
     * Support remote interface for clearing out the index queue.
     */
    public boolean clearIndexQueue() throws RemoteException
    {
        soapServiceHelper.assertCanAdminister();
        indexManager.resetIndexQueue();
        return true;
    }

    public RemoteClusterInformation getClusterInformation()
    {
        return new RemoteClusterInformation(clusterManager.getClusterInformation());
    }

    public RemoteNodeStatus[] getClusterNodeStatuses()
    {
        List l = new ArrayList();
        Map m = NodeStatusFactory.getNodeStatusesForCluster(clusterManager);
        for (Iterator i = m.entrySet().iterator(); i.hasNext();)
        {
            Map.Entry e = (Map.Entry) i.next();
            Member member = (Member) e.getKey();
            NodeStatus nodeStatus = (NodeStatus) e.getValue();
            l.add(new RemoteNodeStatus(member.getId(), nodeStatus));
        }
        return (RemoteNodeStatus[]) l.toArray(new RemoteNodeStatus[0]);
    }

    public void setClusterManager(ClusterManager clusterManager)
    {
        this.clusterManager = clusterManager;
    }
}
